#!/usr/bin/env python

import unittest
from pagerator import IterableQuery

class Tests(unittest.TestCase):
    def setUp(self, *args, **kwargs):
        super(Tests, self).setUp(*args, **kwargs)

        self.hits = []
        def getter(page):
            self.hits.append(page)
            if page == 0:
                return range(0, 10)
            elif page == 1:
                return range(10, 20)
            return []

        self.obj = IterableQuery(getter = getter, page_size = 10)

    def test_chaining(self):
        self.assertEqual(range(0, 20), list(self.obj))

    def test_limiting(self):
        self.assertEqual(range(0, 5), list(self.obj[:5]))
        self.assertEqual(range(5, 20), list(self.obj[5:]))
        self.assertEqual(range(3, 5), list(self.obj[3:5]))
        self.assertEqual(range(5, 15), list(self.obj[5:15]))

    def test_sublimiting(self):
        self.assertEqual(range(5, 15), list(self.obj[:15][5:]))
        self.assertEqual(range(7, 15), list(self.obj[5:15][2:20]))
        self.assertEqual(range(7, 17), list(self.obj[5:30][2:10]))

    def test_skip_pages(self):
        self.assertEqual(range(15, 17), list(self.obj[15:17]))
        self.assertEqual([1], self.hits)


if __name__ == '__main__':
    unittest.main()




import math

def _iterate_over_pages(getter, page_size, start_from, upper_bound):
    page = int(math.floor(start_from / page_size))

    if upper_bound is not None:
        num_items = upper_bound - start_from

    start_from -= page * page_size

    results = getter(page)
    while results:
        for item in results:
            if start_from > 0:
                start_from -= 1
            else:
                yield item
                if upper_bound is not None:
                    num_items -= 1
                    if num_items == 0:
                        return
        page += 1
        results = getter(page)


class IterableQuery(object):
    def __init__(self, getter, page_size, start_from = 0, upper_bound = None):
        self._getter = getter
        self._page_size = page_size
        self._start_from = start_from
        self._upper_bound = upper_bound

    def __iter__(self):
        return _iterate_over_pages(
            self._getter,
            self._page_size,
            self._start_from,
            self._upper_bound,
        )

    def __getitem__(self, key):
        if isinstance(key, slice):
            start = self._start_from
            if key.start is not None:
                if key.start < 0:
                    raise KeyError('Negative indexes aren\'t not supported')
                start += key.start

            if key.stop is None:
                upper_bound = self._upper_bound
            else:
                if self._upper_bound is None:
                    upper_bound = self._start_from + key.stop
                else:
                    upper_bound = min(self._upper_bound, start + key.stop)

            return IterableQuery(
                self._getter,
                self._page_size,
                start_from = start,
                upper_bound = upper_bound,
            )
        raise KeyError('Key %r unsupported.' % (key,))




#!/usr/bin/env python
"""
This example shows how to iterate over last 100 django commits from the github.
"""

import urllib
import simplejson
import pagerator

def get_page(page):
    page -= 1 # on the github, pages numbers begin from 1
    data = urllib.urlopen('http://github.com/api/v2/json/commits/list/django/django/master?page=%s' % page).read()
    json = simplejson.loads(data)
    return json['commits']

def show_last_n(limit):
    all_commits = pagerator.IterableQuery(get_page, 35)
    for commit in all_commits[:100]:
        print '======= %s ======' % commit['id']
        print commit['message']
        print ''


if __name__ == '__main__':
    show_last_n(100)



from setuptools import setup, find_packages

setup(
    name = 'pagerator',
    version = '0.1.0',
    description = 'A helper to build iterables when you need to fetch results already divided into the pages.',
    author = 'Alexander Artemenko',
    author_email = 'svetlyak.40wt@gmail.com',
    keywords = 'helper utility',
    license = 'New BSD License',
    url = 'http://github.com/svetlyak40wt/pagerator/',
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Programming Language :: Python',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],
    packages = find_packages(),
)
